﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SiteMapEditor.Controls
{
    public partial class DescriptionControl : UserControl, ISiteMapSavable
    {
        private Dictionary<string, string> collec;

        private string initialLCID = "";
        private string initialDescription = "";

        #region Delegates

        public delegate void SaveEventHandler(object sender, SaveEventArgs e);

        #endregion

        #region Event Handlers

        public event SaveEventHandler Saved;

        #endregion

        public DescriptionControl()
        {
            InitializeComponent();

            this.collec = new Dictionary<string, string>();

            ToolTip tip = new ToolTip();
            tip.ToolTipTitle = "Information";
            tip.SetToolTip(this.txtDescriptionLCID, "A four or five digit locale ID for the Description.");
            tip.SetToolTip(this.txtDescriptionDescription, "Text to be displayed.");
        }

        public DescriptionControl(Dictionary<string, string> collection):this()
        {
            if(collection != null)
                this.collec = collection;

            this.FillControls();
        }

        private void FillControls()
        {
            this.txtDescriptionLCID.Text = collec.ContainsKey("LCID") ? collec["LCID"] : "";
            this.txtDescriptionDescription.Text = collec.ContainsKey("Description") ? collec["Description"] : "";

            this.initialLCID = this.txtDescriptionLCID.Text;
            this.initialDescription = this.txtDescriptionDescription.Text;
        }

        private void SiteMapControl_Leave(object sender, EventArgs e)
        {
            if (this.initialLCID != this.txtDescriptionLCID.Text ||
                this.initialDescription != this.txtDescriptionDescription.Text)
            {
                if (MessageBox.Show("You didn't save your changes! Do you want to save them now?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    this.Save();
                }
            }
        }

        public void Save()
        {
            int result;
            if (this.txtDescriptionLCID.Text.Length != 4 && this.txtDescriptionLCID.Text.Length != 5 && !int.TryParse(this.txtDescriptionLCID.Text, out result))
            {
                MessageBox.Show(this, "LCID accepts only 4 or 5 digits figure", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            else if (this.txtDescriptionDescription.Text.Length == 0)
            {
                MessageBox.Show(this, "Description can't be empty", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            else
            {
                Dictionary<string, string> collection = new Dictionary<string, string>();
                if (this.txtDescriptionLCID.Text.Length > 0)
                    collection.Add("LCID", this.txtDescriptionLCID.Text);
                if (this.txtDescriptionDescription.Text.Length > 0)
                    collection.Add("Description", this.txtDescriptionDescription.Text);

                this.initialLCID = this.txtDescriptionLCID.Text;
                this.initialDescription = this.txtDescriptionDescription.Text;

                this.SendSaveMessage(collection);
            }
        }

        #region Send Events

        /// <summary>
        /// Sends a connection success message 
        /// </summary>
        /// <param name="service">IOrganizationService generated</param>
        /// <param name="parameters">Lsit of parameter</param>
        private void SendSaveMessage(Dictionary<string, string> collection)
        {
            SaveEventArgs sea = new SaveEventArgs() { AttributeCollection = collection };

            if (this.Saved != null)
            {
                this.Saved(this, sea);
            }
        }

        #endregion

    }
}
