﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SiteMapEditor.Controls
{
    public partial class SiteMapControl : UserControl, ISiteMapSavable
    {
        private Dictionary<string, string> collec;

        private string initialUrl = "";

        #region Delegates

        public delegate void SaveEventHandler(object sender, SaveEventArgs e);

        #endregion

        #region Event Handlers

        public event SaveEventHandler Saved;

        #endregion

        public SiteMapControl()
        {
            InitializeComponent();

            this.collec = new Dictionary<string, string>();

            ToolTip tip = new ToolTip();
            tip.ToolTipTitle = "Information";
            tip.SetToolTip(this.txtSiteMapUrl, "Specifies the URL for Microsoft Dynamics CRM for Outlook to render.");
        }

        public SiteMapControl(Dictionary<string, string> collection):this()
        {

            this.collec = collection;

            this.FillControls();
        }

        private void FillControls()
        {
            this.txtSiteMapUrl.Text = collec.ContainsKey("Url") ? collec["Url"] : "";
            this.initialUrl = this.txtSiteMapUrl.Text;
        }

        private void SiteMapControl_Leave(object sender, EventArgs e)
        {
            if (this.initialUrl != this.txtSiteMapUrl.Text)
            {
                if (MessageBox.Show("You didn't save your changes! Do you want to save them now?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    this.Save();
                }
            }
        }

        public void Save()
        {
            Dictionary<string, string> collection = new Dictionary<string, string>();

            if (this.txtSiteMapUrl.Text.Length > 0)
                collection.Add("Url", this.txtSiteMapUrl.Text);

            this.initialUrl = this.txtSiteMapUrl.Text;

            this.SendSaveMessage(collection);
        }

        #region Send Events

        /// <summary>
        /// Sends a connection success message 
        /// </summary>
        /// <param name="service">IOrganizationService generated</param>
        /// <param name="parameters">Lsit of parameter</param>
        private void SendSaveMessage(Dictionary<string, string> collection)
        {
            SaveEventArgs sea = new SaveEventArgs() { AttributeCollection = collection };

            if (this.Saved != null)
            {
                this.Saved(this, sea);
            }
        }

        #endregion

    }
}
