﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SiteMapEditor.Controls
{
    public partial class TitleControl : UserControl, ISiteMapSavable
    {
        private Dictionary<string, string> collec;

        private string initialLCID = "";
        private string initialTitle = "";

        #region Delegates

        public delegate void SaveEventHandler(object sender, SaveEventArgs e);

        #endregion

        #region Event Handlers

        public event SaveEventHandler Saved;

        #endregion

        public TitleControl()
        {
            InitializeComponent();

            this.collec = new Dictionary<string, string>();

            ToolTip tip = new ToolTip();
            tip.ToolTipTitle = "Information";
            tip.SetToolTip(this.txtTitleLCID, "A four or five digit Locale ID for the title.");
            tip.SetToolTip(this.txtTitleTitle, "Text to be displayed.");
        }

        public TitleControl(Dictionary<string, string> collection)
            : this()
        {
            if(collection != null)
                this.collec = collection;

            this.FillControls();
        }

        private void FillControls()
        {
            this.txtTitleLCID.Text = collec.ContainsKey("LCID") ? collec["LCID"] : "";
            this.txtTitleTitle.Text = collec.ContainsKey("Title") ? collec["Title"] : "";

            this.initialLCID = this.txtTitleLCID.Text;
            this.initialTitle = this.txtTitleTitle.Text;
        }

        private void SiteMapControl_Leave(object sender, EventArgs e)
        {
            if (this.initialLCID != this.txtTitleLCID.Text ||
                this.initialTitle != this.txtTitleTitle.Text)
            {
                if (MessageBox.Show("You didn't save your changes! Do you want to save them now?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    this.Save();
                }
            }
        }

        public void Save()
        {
            int result;
            if (this.txtTitleLCID.Text.Length != 4 && this.txtTitleLCID.Text.Length != 5 && !int.TryParse(this.txtTitleLCID.Text, out result))
            {
                MessageBox.Show(this, "LCID accepts only 4 or 5 digits figure", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            else if (this.txtTitleLCID.Text.Length == 0)
            {
                MessageBox.Show(this, "Title can't be empty", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            else
            {
                Dictionary<string, string> collection = new Dictionary<string, string>();
                if (this.txtTitleLCID.Text.Length > 0)
                    collection.Add("LCID", this.txtTitleLCID.Text);
                if (this.txtTitleTitle.Text.Length > 0)
                    collection.Add("Title", this.txtTitleTitle.Text);

                this.initialLCID = this.txtTitleLCID.Text;
                this.initialTitle = this.txtTitleTitle.Text;

                this.SendSaveMessage(collection);
            }
        }

        #region Send Events

        /// <summary>
        /// Sends a connection success message 
        /// </summary>
        /// <param name="service">IOrganizationService generated</param>
        /// <param name="parameters">Lsit of parameter</param>
        private void SendSaveMessage(Dictionary<string, string> collection)
        {
            SaveEventArgs sea = new SaveEventArgs() { AttributeCollection = collection };

            if (this.Saved != null)
            {
                this.Saved(this, sea);
            }
        }

        #endregion

    }
}
