﻿using System;
using System.Windows.Forms;
using System.Xml;
using System.Reflection;
using System.IO;

namespace SiteMapEditor.Forms
{
    public partial class SiteMapComponentPicker : Form
    {
        public XmlNode SelectedNode { get; set; }

        public SiteMapComponentPicker(string componentName)
        {
            InitializeComponent();

            XmlDocument doc = new XmlDocument();
            
            Assembly myAssembly = Assembly.GetExecutingAssembly();
            using (StreamReader reader = new StreamReader(myAssembly.GetManifestResourceStream("SiteMapEditor.Resources.sitemap.xml")))
            {
                doc.LoadXml(reader.ReadToEnd());
            }
            
            FillList(doc, componentName);

            ToolTip tip = new ToolTip();
            tip.ToolTipTitle = "Information";
            tip.SetToolTip(this.chkAddChildComponents, "Check this control if you want to add components under the one you select (ie. Area with all child Groups and SubArea or just Area)");
        }

        private void FillList(XmlDocument doc, string componentName)
        {
            try
            {
                XmlNodeList list = doc.SelectSingleNode("ImportExportXml/SiteMap/SiteMap").SelectNodes("//" + componentName);

                foreach (XmlNode node in list)
                {
                    if (!this.lstComponents.Items.ContainsKey(node.Attributes["Id"].Value))
                    {
                        ListViewItem item = new ListViewItem(node.Attributes["Id"].Value);
                        item.SubItems.Add(node.Attributes["Entity"] != null ? node.Attributes["Entity"].Value : "-");
                        item.SubItems.Add(node.Attributes["ResourceId"] != null ? node.Attributes["ResourceId"].Value : "-");
                        item.Tag = node;

                        if (node.ParentNode != null)
                        {
                            string groupName = node.ParentNode.Name;

                            if (node.ParentNode.Attributes["Id"] != null)
                            {
                                groupName += " (" + node.ParentNode.Attributes["Id"].Value + ")";
                            }

                            ListViewGroup group = this.lstComponents.Groups[groupName.Replace(" ", "")];

                            if (group == null)
                            {
                                group = new ListViewGroup(groupName);
                                group.Name = groupName.Replace(" ", "");
                                this.lstComponents.Groups.Add(group);
                            }

                            item.Group = group;
                        }

                        this.lstComponents.Items.Add(item);
                    }
                }
            }
            catch (Exception error)
            {
                MessageBox.Show(this, error.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);            
            }
        }

        private void lstComponents_DoubleClick(object sender, EventArgs e)
        {
            this.btnComponentPickerValidate_Click(sender, e);
        }

        private void btnComponentPickerValidate_Click(object sender, EventArgs e)
        {
            if (this.lstComponents.SelectedItems.Count > 0)
            {
                XmlNode selectedXmlNode = (XmlNode)this.lstComponents.SelectedItems[0].Tag;

                if (!this.chkAddChildComponents.Checked)
                {
                    for (int i = selectedXmlNode.ChildNodes.Count - 1; i >= 0; i--)
                    {
                        selectedXmlNode.RemoveChild(selectedXmlNode.ChildNodes[i]);
                    }
                }

                this.SelectedNode = selectedXmlNode;
                this.DialogResult = DialogResult.OK;
                this.Close();
            }
            else
            {
                MessageBox.Show(this, "Please select a component!", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void btnComponentPickerCancel_Click(object sender, EventArgs e)
        {
            this.SelectedNode = null;
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }
    }
}
